@@ -22,7 +22,7 @@ use Encode ();
 use MIME::Base64;
 use Scalar::Util ();
 
-our $VERSION = 1.8;
+our $VERSION = 1.9;
 
 my $apiVersion = undef;  # 0 for compatibility.  1 for prefered
 my $error;
@@ -287,9 +287,9 @@ sub _listOIDs {
 sub setAPIversion {
     my( $class, $version ) = @_;
 
-    croak( ($error = "Wrong number of arguments\n") ) unless( @_ == 2 && defined $class );
+    croak( substr(($error = "Wrong number of arguments\n"), 0, -1) ) unless( @_ == 2 && defined $class && !ref $class );
     $version = 0 unless( defined $version );
-    croak( ($error = "Unsupported API version $version\n") ) unless( $version >= 0 && $version <= 1 );
+    croak( substr(($error = "Unsupported API version $version\n"), 0, -1) ) unless( $version >= 0 && $version <= 1 );
     $apiVersion = $version;
 
     $version = !$version || 0;
@@ -305,7 +305,9 @@ sub setAPIversion {
 sub getAPIversion {
     my( $class ) = @_;
 
-    croak( "Class not specified for getAPIversion()\n" ) unless( defined $class );
+    croak( "Class not specified for getAPIversion()" ) unless( defined $class );
+
+    return $class->{_apiVersion} if( ref $class && $class->isa( __PACKAGE__ ) );
 
     return $apiVersion;
 }
@@ -314,7 +316,7 @@ sub name2oid {
     my $class = shift;
     my( $oid ) = @_;
 
-    croak( "Class not specifed for name2oid()\n" ) unless( defined $class );
+    croak( "Class not specifed for name2oid()" ) unless( defined $class );
 
     return unless( defined $oid && defined $apiVersion && $apiVersion > 0 );
 
@@ -325,7 +327,7 @@ sub oid2name {
     my $class = shift;
     my( $oid ) = @_;
 
-    croak( "Class not specifed for oid2name()\n" ) unless( defined $class );
+    croak( "Class not specifed for oid2name()" ) unless( defined $class );
 
     return $oid unless( defined $apiVersion && $apiVersion > 0 );
 
@@ -354,10 +356,10 @@ sub _oid2name {
 sub registerOID {
     my( $class, $oid, $longname, $shortname ) = @_;
 
-    croak( "Class not specifed for registerOID()\n" ) unless( defined $class );
+    croak( "Class not specifed for registerOID()" ) unless( defined $class );
 
     unless( defined $apiVersion ) {
-	carp( "${class}::setAPIversion MUST be called before registerOID().  Defaulting to legacy mode\n" );
+	carp( "${class}::setAPIversion MUST be called before registerOID().  Defaulting to legacy mode" );
 	$class->setAPIversion(0);
     }
 
@@ -390,12 +392,24 @@ sub new {
 
     undef $error;
 
+    $class = ref $class if( defined $class && ref $class && $class->isa( __PACKAGE__ ) );
+
+    unless( defined $apiVersion ) {
+	carp( "${class}::setAPIversion MUST be called before new().  Defaulting to legacy mode" );
+	$class->setAPIversion(0);
+    }
+
+    my( $void, $die ) = ( !defined wantarray, 0 );
     my $self = eval {
         die( "Insufficient arguments for new\n" ) unless( defined $class && @_ >= 1 );
-	return $class->_new( @_ );
+        die( "Value of Crypt::PKCS10->new ignored\n" ) if( $void );
+	return $class->_new( \$die, @_ );
     }; if( $@ ) {
 	$error = $@;
-	croak( $@ ) unless( $apiVersion );
+        if( !$apiVersion || $die || !defined wantarray ) {
+            1 while( chomp $@ );
+            croak( $@ );
+        }
 	return;
     }
 
@@ -405,26 +419,27 @@ sub new {
 sub error {
     my $class = shift;
 
-    croak( "Class not specifed for error()\n" ) unless( defined $class );
+    croak( "Class not specifed for error()" ) unless( defined $class );
 
+    if( ref $class && $class->isa( __PACKAGE__ ) ) {
+        return $class->{_error};
+    }
     return $error;
 }
 
-sub _new {
-    my $class  = shift;
-    my $der    = shift;
+my $pemre = qr/(?ms:^\r?-----BEGIN\s(?:NEW\s)?CERTIFICATE\sREQUEST-----\s*\r?\n\s*(.*?)\s*^\r?-----END\s(?:NEW\s)?CERTIFICATE\sREQUEST-----\r?$)/;
 
-    unless( defined $apiVersion ) {
-	carp( "${class}::setAPIversion MUST be called before new().  Defaulting to legacy mode\n" );
-	$class->setAPIversion(0);
-    }
+sub _new {
+    my( $class, $die, $der ) = splice( @_, 0, 3 );
 
     my %options = (
                    acceptPEM       => 1,
+                   PEMonly         => 0,
                    escapeStrings   => 1,
                    readFile        => 0,
                    ignoreNonBase64 => 0,
-                   verifySignature => $apiVersion,
+                   verifySignature => ($apiVersion >= 1),
+                   dieOnError      => 0,
                   );
 
     %options = ( %options, %{ shift @_ } ) if( @_ >= 1 && ref( $_[0] ) eq 'HASH' );
@@ -433,37 +448,56 @@ sub _new {
 
     %options = ( %options, @_ ) if( @_ );
 
-    my $self = {};
+    my $self = { _apiVersion => $apiVersion };
+
+    my $keys = join( '|', qw/escapeStrings acceptPEM PEMonly binaryMode readFile verifySignature ignoreNonBase64 warnings dieOnError/ );
+
+    $self->{"_$_"} = delete $options{$_} foreach (grep { /^(?:$keys)$/ } keys %options);
+
+    $$die = $self->{_dieOnError} &&= $apiVersion >= 1;
+
+    die( "\$csr argument to new() is not defined\n" ) unless( defined $der );
 
-    $self->{"_$_"} = delete $options{$_} foreach (grep { /^(?:escapeStrings|acceptPEM|readFile|verifySignature|ignoreNonBase64|warnings)$/ } keys %options);
     if( keys %options ) {
 	die( "Invalid option(s) specified: " . join( ', ', sort keys %options ) . "\n" );
     }
 
+    $self->{_binaryMode} = !$self->{_acceptPEM} unless( exists $self->{_binaryMode} );
+
     my $parser;
 
     # malformed requests can produce various warnings; don't proceed in that case.
 
-    local $SIG{__WARN__} = sub { my $msg = $_[0]; $msg =~ s/\A(.*?) at .*\Z/$1/s; chomp $msg; die "$msg\n" };
+    local $SIG{__WARN__} = sub { my $msg = $_[0]; $msg =~ s/\A(.*?) at .*\Z/$1/s; 1 while( chomp $msg ); die "$msg\n" };
 
     if( $self->{_readFile} ) {
-        open( my $fh, '<', $der ) or croak( "Failed to open $der: $!" );
+        open( my $fh, '<', $der ) or die( "Failed to open $der: $!\n" );
         $der = $fh;
     }
 
     if( Scalar::Util::openhandle( $der ) ) {
 	local $/;
 
-	binmode $der unless( $self->{_acceptPEM} );
+	binmode $der if( $self->{_binaryMode} );
 
 	$der = <$der>;          # Note: this closes files opened by readFile
-	croak( "Failed to read request: $!\n" ) unless( defined $der );
+	die( "Failed to read request: $!\n" ) unless( defined $der );
     }
 
-    if( $self->{_acceptPEM} && $der =~         # if PEM, convert to DER
-/^\r?-----BEGIN\s(?:NEW\s)?CERTIFICATE\sREQUEST-----\r?\n\s*(.*?)\s*^\r?-----END\s(?:NEW\s)?CERTIFICATE\sREQUEST-----\r?$/ms ) {
-	$der = $1;
+    my $isPEM;
 
+    if( $self->{_PEMonly} ) {
+        if( $der =~ $pemre ) {
+            $der = $1;
+            $isPEM = 1;
+        } else {
+            die( "No certificate request found\n" );
+        }
+    } elsif( $self->{_acceptPEM} && $der =~ $pemre ) {
+        $der = $1;
+        $isPEM = 1;
+    }
+    if( $isPEM ) {
 	# Some versions of MIME::Base64 check the input.  Some don't.  Those that do
 	# seem to obey -w, but not 'use warnings'.  So we'll check here.
 
@@ -498,7 +532,10 @@ sub _new {
                            $tlen, $tlen - $len ) );
         }
         return substr( $der, 0, $len );
-    }; croak( "Invalid format for request: $@\n" ) if( $@ );
+    }; if( $@ ) {
+        1 while( chomp $@ );
+        die( "Invalid format for request: $@\n" );
+    }
 
     $self->{_der} = $der;
 
@@ -508,7 +545,7 @@ sub _new {
 
     my $asn = Convert::ASN1->new;
     $self->{_asn} = $asn;
-    $asn->prepare(<<ASN1) or croak( $asn->error );
+    $asn->prepare(<<ASN1) or die( "Internal error in " . __PACKAGE__ . ": " . $asn->error );
 
     DirectoryString ::= CHOICE {
       teletexString   TeletexString,
@@ -733,17 +770,17 @@ ASN1
 
     my( $CRtaglen, $CRtag, $CRllen, $CRlen );
     ($CRtaglen, undef, $CRtag) = asn_decode_tag2( $der );
-    croak( "Invalid CSR format" ) unless( $CRtag == ASN_SEQUENCE );
+    die( "Invalid CSR format: missing SEQUENCE 1\n" ) unless( $CRtag == ASN_SEQUENCE );
     ($CRllen, $CRlen) = asn_decode_length( substr( $der, $CRtaglen ) );
 
     my( $CItaglen, $CItag, $CIllen, $CIlen );
     ($CItaglen, undef, $CItag) = asn_decode_tag2( substr( $der, $CRtaglen + $CRllen ) );
-    croak( "Invalid CSR format" ) unless( $CItag == ASN_SEQUENCE );
+    die( "Invalid CSR format: missing SEQUENCE 2\n" ) unless( $CItag == ASN_SEQUENCE );
     ($CIllen, $CIlen) = asn_decode_length( substr( $der, $CRtaglen + $CRllen + $CItaglen ) );
 
     $self->{_signed} = substr( $der, $CRtaglen +  $CRllen, $CItaglen + $CIllen + $CIlen );
 
-    croak( $error ) if( $self->{_verifySignature} && !$self->checkSignature );
+    die( $error ) if( $self->{_verifySignature} && !$self->checkSignature );
 
     return $self;
 }
@@ -858,7 +895,7 @@ my %special;
 
      @usages = @usages[ grep { $bit & (1 << $_ - $shift) } 0 .. $#usages ]; #transfer bitmap to barewords
 
-     return [ @usages ] if( $apiVersion >= 1 );
+     return [ @usages ] if( $self->{_apiVersion} >= 1 );
 
      return join( ', ', @usages );
  },
@@ -872,7 +909,7 @@ my %special;
      return unpack( "H*", $value );
  },
  ApplicationCertPolicies => sub {
-     goto &{$special{certificatePolicies}} if( $apiVersion > 0 );
+     goto &{$special{certificatePolicies}} if( $_[0]->{_apiVersion} > 0 );
 
      my $self = shift;
      my( $value, $id ) = @_;
@@ -887,7 +924,7 @@ my %special;
      my $self = shift;
      my( $value, $id ) = @_;
 
-     $value->{templateID} = $self->_oid2name( $value->{templateID} ) if( $apiVersion > 0 );
+     $value->{templateID} = $self->_oid2name( $value->{templateID} ) if( $self->{_apiVersion} > 0 );
      return $value;
  },
  ENROLLMENT_NAME_VALUE_PAIR => sub {
@@ -945,7 +982,7 @@ my %special;
      my $self = shift;
      my( $value, $id, $entry ) = @_;
 
-     return $self->_convert_extensionRequest( [ $value ] ) if( $apiVersion > 0 ); # Untested
+     return $self->_convert_extensionRequest( [ $value ] ) if( $self->{_apiVersion} > 0 ); # Untested
  },
  BasicConstraints => sub {
      my $self = shift;
@@ -1081,7 +1118,7 @@ sub _init {
     my $parsed = $self->{_asn}->find($node);
 
     unless( defined $parsed || $optional ) {
-	croak( "Missing node $node in ASN.1\n" );
+	croak( "Missing node $node in ASN.1" );
     }
     return $parsed;
 }
@@ -1187,15 +1224,35 @@ sub subjectPublicKeyParams {
     my $self = shift;
     my $detail = shift;
 
-    croak( "Requires API version 1\n" ) unless( $apiVersion >= 1 );
+    croak( "Requires API version 1" ) unless( $self->{_apiVersion} >= 1 );
+
+    undef $error;
+    delete $self->{_error};
 
     my $rv = {};
     my $at = $self->pkAlgorithm;
-    if( $at eq 'ecPublicKey' ) {
+    $at = 'undef' unless( defined $at );
+
+    if( $at eq 'rsaEncryption' ) {
+        $rv->{keytype} = 'RSA';
+        my $par = $self->_init( 'rsaKey' );
+        my $rsa = $par->decode( $self->{certificationRequestInfo}{subjectPKInfo}{subjectPublicKey}[0] );
+        $rv->{keylen} = 4 * ( length( $rsa->{modulus}->as_hex ) -2 ); # 2 == length( '0x' )
+        $rv->{modulus} = substr( $rsa->{modulus}->as_hex, 2 );
+        $rv->{publicExponent} = ( ref( $rsa->{publicExponent} )?
+                                  $rsa->{publicExponent}->as_hex :
+                                  sprintf( '%x', $rsa->{publicExponent} ) );
+    } elsif( $at eq 'ecPublicKey' ) {
         $rv->{keytype} = 'ECC';
 
-        require Crypt::PK::ECC;
-
+        eval { require Crypt::PK::ECC; };
+        if( $@ ) {
+            $rv->{keytype} = undef;
+            $self->{_error} =
+              $error = "ECC public key requires Crypt::PK::ECC\n";
+            croak( $error ) if( $self->{_dieOnError} );
+            return $rv;
+        }
         my $key = $self->subjectPublicKey(1);
         $key = Crypt::PK::ECC->new( \$key )->key2hash;
         $rv->{keylen} = $key->{curve_bits};
@@ -1206,15 +1263,6 @@ sub subjectPublicKeyParams {
         my $par = $self->_init( 'eccName' );
         $rv->{curve} = $par->decode( $self->{certificationRequestInfo}{subjectPKInfo}{algorithm}{parameters} );
         $rv->{curve} = $self->_oid2name( $rv->{curve} );
-    } elsif( $at eq 'rsaEncryption' ) {
-        $rv->{keytype} = 'RSA';
-        my $par = $self->_init( 'rsaKey' );
-        my $rsa = $par->decode( $self->{certificationRequestInfo}{subjectPKInfo}{subjectPublicKey}[0] );
-        $rv->{keylen} = 4 * ( length( $rsa->{modulus}->as_hex ) -2 ); # 2 == length( '0x' )
-        $rv->{modulus} = substr( $rsa->{modulus}->as_hex, 2 );
-        $rv->{publicExponent} = ( ref( $rsa->{publicExponent} )?
-                                  $rsa->{publicExponent}->as_hex :
-                                  sprintf( '%x', $rsa->{publicExponent} ) );
     } elsif( $at eq 'dsa' ) {
         $rv->{keytype} = 'DSA';
         my $par = $self->_init( 'dsaKey' );
@@ -1229,6 +1277,9 @@ sub subjectPublicKeyParams {
         }
     } else {
         $rv->{keytype} = undef;
+        $self->{_error} =
+          $error = "Unrecognized public key type $at\n";
+        croak( $error ) if( $self->{_dieOnError} );
     }
     return $rv;
 }
@@ -1288,7 +1339,7 @@ sub attributes {
     my $self = shift;
     my( $name ) = @_;
 
-    if( $apiVersion < 1 ) {
+    if( $self->{_apiVersion} < 1 ) {
 	my $attributes = $self->{certificationRequestInfo}{attributes};
 	return () unless( defined $attributes );
 
@@ -1410,7 +1461,7 @@ sub extensions {
     return () unless( defined $attributes && exists $attributes->{extensionRequest} );
 
     my @present =  map { $_->{extnID} } @{$attributes->{extensionRequest}};
-    if( $apiVersion >= 1 ) {
+    if( $self->{_apiVersion} >= 1 ) {
 	foreach my $ext (@present) {
 	    $ext = $variantNames{'$' . $ext} if( exists $variantNames{'$' . $ext} );
 	}
@@ -1433,7 +1484,7 @@ sub extensionValue {
     foreach my $entry (@{$attributes->{extensionRequest}}) {
         if ($entry->{extnID} eq $extensionName) {
             $value = $entry->{extnValue};
-	    if( $apiVersion == 0 ) {
+	    if( $self->{_apiVersion} == 0 ) {
 		while (ref $value eq 'HASH') {
 		    my @keys = sort keys %{$value};
 		    $value = $value->{ shift @keys } ;
@@ -1478,9 +1529,10 @@ sub checkSignature {
     my $self = shift;
 
     undef $error;
+    delete $self->{_error};
 
     my $ok = eval {
-        croak( "checkSignature requires API version 1\n" ) unless( $apiVersion >= 1 );
+        die( "checkSignature requires API version 1\n" ) unless( $self->{_apiVersion} >= 1 );
 
         my $key = $self->subjectPublicKey(1); # Key as PEM
         my $sig = $self->signature(1);        # Signature as DER
@@ -1498,49 +1550,57 @@ sub checkSignature {
             $hashmod = "Digest::MD$1";
             $hashfcn = "Digest::MD$1::$hash";
         } else {
-            croak( "Unknown hash in signature algorithm $alg\n" );
+            die( "Unknown hash in signature algorithm $alg\n" );
         }
 
         my $keyp = $self->subjectPublicKeyParams;
 
-        croak( "Unknown public key type\n" ) unless( defined $keyp->{keytype} );
+        die( "Unknown public key type\n" ) unless( defined $keyp->{keytype} );
 
         # Verify signature using the correct module and hash type.
 
         if( $keyp->{keytype} eq 'RSA' ) {
-            require Crypt::OpenSSL::RSA;
+            eval { require Crypt::OpenSSL::RSA; };
+            die( "Unable to load Crypt::OpenSSL::RSA\n") if( $@ );
 
             $key = Crypt::OpenSSL::RSA->new_public_key( $key );
             $hash = "use_${hash}_hash";
-            $key->$hash;
+            eval { $key->$hash; };
+            die( "Unsupported hash type $hash\n" ) if( $@ );
             $key->use_pkcs1_padding;
             return $key->verify( $self->certificationRequest, $sig );
         }
 
         if( $keyp->{keytype} eq 'DSA' ) {
-            require Crypt::OpenSSL::DSA;
-            eval "require $hashmod" or croak( $@ ); ## no critic (BuiltinFunctions::ProhibitStringyEval)
+            eval { require Crypt::OpenSSL::DSA; };
+            die( "Unable to load Crypt::OpenSSL::DSA\n" ) if( $@ );
+            eval "require $hashmod" or die( "Unable to load $hashmod\n" ); ## no critic (BuiltinFunctions::ProhibitStringyEval)
 
             my $dsa = Crypt::OpenSSL::DSA->read_pub_key_str( $key );
             return $dsa->verify( eval "$hashfcn( \$self->certificationRequest )", $sig ); ## no critic (BuiltinFunctions::ProhibitStringyEval)
         }
 
         if( $keyp->{keytype} eq 'ECC' ) {
-            require Crypt::PK::ECC;
+            eval { require Crypt::PK::ECC; };
+            die( "Unable to load Crypt::PK::ECC\n" ) if( $@ );
 
             $key = Crypt::PK::ECC->new( \$key );
             return $key->verify_message( $sig, $self->certificationRequest, uc($hash) );
         }
 
-        croak( "Unknown key type $keyp->{keytype}\n" );
+        die( "Unknown key type $keyp->{keytype}\n" );
     };
     if( $@ ) {
-        $error = $@;
+        $self->{_error} =
+          $error = $@;
+        croak( $error ) if( $self->{_dieOnError} );
         return;
     }
     return 1 if( $ok );
 
-    $error = "Incorrect signature";
+    $self->{_error} =
+      $error = "Incorrect signature\n";
+    croak( $error ) if( $self->{_dieOnError} );
 
     return 0;
 }
@@ -1574,13 +1634,16 @@ sub as_string {
     local( $@, $_, $! );
 
     my $v = $apiVersion;
-    $self->setAPIversion( 1 ) unless( defined $v && $v == 1 );
+    ref( $self )->setAPIversion( 1 ) unless( defined $v && $v == 1 );
 
     my $string = eval {
+        $self = ref( $self )->new( $self->{_der}, acceptPEM => 0, verifySignature => 0, escapeStrings => 0 );
+        return $error if( !defined $self );
+
         $self->__stringify;
     };
     my $at = $@;
-    $self->setAPIversion( $v ) unless( defined $v && $v == 1 );
+    ref( $self )->setAPIversion( $v ) unless( defined $v && $v == 1 );
 
     $string = '' unless( defined $string );
     $string .= $at if( $at );
@@ -1630,7 +1693,7 @@ sub __stringify {
     foreach (sort keys %$kp) {
         my $v = $kp->{$_};
 
-        if( !defined $v ) {
+        if( !defined $v && !defined( $v = $self->error ) ) {
             $v = 'undef';
         } elsif( ref $v ) {
             next;
@@ -1673,44 +1736,10 @@ sub __stringify {
 
 __END__
 
+=encoding utf-8
+
 =pod
 
-=encoding UTF-8
-
-=head1 NAME
-
-Crypt::PKCS10 - parse PKCS #10 certificate requests
-
-=head1 VERSION
-
-version 1.8
-
-=head1 SYNOPSIS
-
-    use Crypt::PKCS10;
-
-    Crypt::PKCS10->setAPIversion( 1 );
-    my $decoded = Crypt::PKCS10->new( $csr ) or die Crypt::PKCS10->error;
-
-    print $decoded;
-
-    @names = $decoded->extensionValue('subjectAltName' );
-    @names = $decoded->subject unless( @names );
-
-    %extensions = map { $_ => $decoded->extensionValue( $_ ) } $decoded->extensions
-
-=head1 DESCRIPTION
-
-C<Crypt::PKCS10> parses PKCS #10 certificate requests (CSRs) and provides accessor methods to extract the data in usable form.
-
-Common object identifiers will be translated to their corresponding names.
-Additionally, accessor methods allow extraction of single data fields.
-The format of returned data varies by accessor.
-
-The access methods return the value corresponding to their name.  If called in scalar context, they return the first value (or an empty string).  If called in array context, they return all values.
-
-B<true> values should be specified as 1 and B<false> values as 0.  Future API changes may provide different functions when other values are used.
-
 =begin :readme
 
   This file is automatically generated by pod2readme from PKCS10.pm and Changes.
@@ -1750,6 +1779,12 @@ F<Changes> describes additional improvements.  Details follow.
 
 =head1 INSTALLATION
 
+C<Crypt::PKCS10> supports DSA, RSA and ECC public keys in CSRs.
+
+It depends on C<Crypt::OpenSSL::DSA>, C<Crypt::OpenSSL::RSA> and C<Crypt::PK::ECC>
+for some operations.  All are recommended.  Some methods will return errors if
+Crypt::PKCS10 is presented with a CSR containing an unsupported public key type.
+
 To install this module type the following:
 
     perl Makefile.PL
@@ -1775,6 +1810,32 @@ Very old CSRs may require C<DIGEST::MD{5,4,2}>
 
 =end :readme
 
+=head1 SYNOPSIS
+
+    use Crypt::PKCS10;
+
+    Crypt::PKCS10->setAPIversion( 1 );
+    my $decoded = Crypt::PKCS10->new( $csr ) or die Crypt::PKCS10->error;
+
+    print $decoded;
+
+    @names = $decoded->extensionValue('subjectAltName' );
+    @names = $decoded->subject unless( @names );
+
+    %extensions = map { $_ => $decoded->extensionValue( $_ ) } $decoded->extensions
+
+=head1 DESCRIPTION
+
+C<Crypt::PKCS10> parses PKCS #10 certificate requests (CSRs) and provides accessor methods to extract the data in usable form.
+
+Common object identifiers will be translated to their corresponding names.
+Additionally, accessor methods allow extraction of single data fields.
+The format of returned data varies by accessor.
+
+The access methods return the value corresponding to their name.  If called in scalar context, they return the first value (or an empty string).  If called in array context, they return all values.
+
+B<true> values should be specified as 1 and B<false> values as 0.  Future API changes may provide different functions when other values are used.
+
 =head1 METHODS
 
 Access methods may exist for subject name components that are not listed here.  To test for these, use code of the form:
@@ -1789,6 +1850,9 @@ Selects the API version (0 or 1) expected.
 
 Must be called before calling any other method.
 
+The API version determines how a CSR is parsed.  Changing the API version after
+parsing a CSR will cause accessors to produce unpredictable results.
+
 =over 4
 
 =item Version 0 - B<DEPRECATED>
@@ -1797,6 +1861,8 @@ Some OID names have spaces and descriptions
 
 This is the format used for C<Crypt::PKCS10> version 1.3 and lower.  The attributes method returns legacy data.
 
+Some new API functions are disabled.
+
 =item Version 1
 
 OID names from RFCs - or at least compatible with OpenSSL and ASN.1 notation.  The attributes method conforms to version 1.
@@ -1811,6 +1877,8 @@ To ease migration, both old and new names are accepted by the API.
 
 Every program should call C<setAPIversion(1)>.
 
+=cut
+
 =head2 class method getAPIversion
 
 Returns the current API version.
@@ -1849,11 +1917,34 @@ If the first option is a HASHREF, it is expanded and any remaining options are a
 
 If B<false>, the input must be in DER format.  C<binmode> will be called on a file handle.
 
-If B<true>, the input is checked for a PEM certificate request.  If not found, the csr
+If B<true>, the input is checked for a C<CERTIFICATE REQUEST> header.  If not found, the csr
 is assumed to be in DER format.
 
 Default is B<true>.
 
+=item PEMonly
+
+If B<true>, the input must be in PEM format.  An error will be returned if the input doesn't contain a C<CERTIFICATE REQUEST> header.
+If B<false>, the input is parsed according to C<acceptPEM>.
+
+Default is B<false>.
+
+=item binaryMode
+
+If B<true>, an input file or file handle will be set to binary mode prior to reading.
+
+If B<false>, an input file or file handle's C<binmode> will not be modified.
+
+Defaults to B<false> if B<acceptPEM> is B<true>, otherwise B<true>.
+
+=item dieOnError
+
+If B<true>, any API function that sets an error string will also C<die>.
+
+If B<false>, exceptions are only generated for fatal conditions.
+
+The default is B<false>.  API version 1 only..
+
 =item escapeStrings
 
 If B<true>, strings returned for extension and attribute values are '\'-escaped when formatted.
@@ -1886,7 +1977,7 @@ The default is B<false>.
 
 =item verifySignature
 
-If B<true>, the CSR's signature is checked.  If verification fails, C<new> will fail.
+If B<true>, the CSR's signature is checked.  If verification fails, C<new> will fail.  Requires API version 1.
 
 If B<false>, the CSR's signature is not checked.
 
@@ -1894,7 +1985,13 @@ The default is B<true> for API version 1 and B<false> for API version 0.
 
 =back
 
-No exceptions are generated.
+No exceptions are generated, unless C<dieOnError> is set or C<new()> is called in
+void context.
+
+The defaults will accept either PEM or DER from a string or file hande, which will
+not be set to binary mode.  Automatic detection of the data format may not be
+reliable on file systems that represent text and binary files differently. Set
+C<acceptPEM> to B<false> and C<PEMonly> to match the file type on these systems.
 
 The object will stringify to a human-readable representation of the CSR.  This is
 useful for debugging and perhaps for displaying a request.  However, the format
@@ -1906,10 +2003,17 @@ can extract.
 If another object inherits from C<Crypt::PKCS10>, it can extend the representation
 by overloading or calling C<as_string>.
 
-=head2 class method error
+=head2 {class} method error
 
 Returns a string describing the last error encountered;
 
+If called as an instance method, last error encountered by the object.
+
+If called as a class method, last error encountered by the class.
+
+Any method can reset the string to B<undef>, so the results are
+only valid immediately after a method call.
+
 =head2 class method name2oid( $oid )
 
 Returns the OID corresponding to a name returned by an access method.
@@ -2021,7 +2125,10 @@ the public key type.
 
 =head3 Standard items:
 
-C<keytype> - ECC, RSA, DSA
+C<keytype> - ECC, RSA, DSA or C<undef>
+
+C<keytype> will be C<undef> if the key type is not supported.  In
+this case, C<error()> returns a diagnostic message.
 
 C<keylen> - Approximate length of the key in bits.
 
@@ -2093,6 +2200,7 @@ In array context, the value(s) are returned as a list of items, which may be ref
  print( " $_: ", scalar $decoded->attributes($_), "\n" )
                                    foreach ($decoded->attributes);
 
+
 =for readme stop
 
 See the I<Table of known OID names> below for a list of names.
@@ -2118,6 +2226,7 @@ The values of extensions vary, however the following code fragment will dump mos
 
  print( "$_: ", $decoded->extensionValue($_,1), "\n" ) foreach ($decoded->extensions);
 
+
 The sample code fragment is not guaranteed to handle all cases.
 Production code needs to select the extensions that it understands and should respect
 the B<critical> boolean.  B<critical> can be obtained with extensionPresent.
@@ -2206,6 +2315,7 @@ To register a shortname for an existing OID without one, specify C<$longname> as
 E.g. To register /E for emailAddress, use:
   C<< Crypt::PKCS10->registerOID( '1.2.840.113549.1.9.1', undef, 'e' ) >>
 
+
 Generates an exception if any argument is not valid, or is in use.
 
 Returns B<true> otherwise.
@@ -2214,13 +2324,14 @@ Returns B<true> otherwise.
 
 Verifies the signature of a CSR.  (Useful if new() specified C<< verifySignature => 0 >>.)
 
-Returns true if the signature is OK.
+Returns B<true> if the signature is OK.
 
-Returns false if the signature is incorrect.  C<< Crypt::PKCS10->error >> returns
+Returns B<false> if the signature is incorrect.  C<< error() >> returns
 the reason.
 
-Returns undef if there was an error in the verification process (e.g. a required
-Perl module could not be loaded.)
+Returns B<undef> if it was not possible to complete the verification process (e.g. a required
+Perl module could not be loaded or an unsupported key/signature type is present.)
+
 
 =head2 certificateTemplate
 
@@ -2238,9 +2349,13 @@ structures, and as names by methods such as B<extensionValue>.
 Unknown OIDs are returned in numeric form, or can be registered with
 B<registerOID>.
 
-=for MAINTAINER  To generate the following table, use:
+=begin MAINTAINER
+
+ To generate the following table, use:
     perl -Mwarnings -Mstrict -MCrypt::PKCS10 -e'Crypt::PKCS10->_listOIDs'
 
+=end MAINTAINER
+
  OID                        Name (API v1)              Old Name (API v0)
  -------------------------- -------------------------- ---------------------------
  0.9.2342.19200300.100.1.1  userID
@@ -2427,38 +2542,13 @@ C<Crypt::PKCS10> is based on the generic ASN.1 module by Graham Barr and on the
  x509decode example by Norbert Klasen. It is also based upon the
 works of Duncan Segrest's C<Crypt-X509-CRL> module.
 
-=for readme start
-
-
 =head1 AUTHORS
 
-=over 4
-
-=item *
-
 Gideon Knocke <gknocke@cpan.org>
-
-=item *
-
 Timothe Litt <tlhackque@cpan.org>
 
-=back
+=head1 LICENSE
 
-=head1 COPYRIGHT AND LICENSE
-
-This software is copyright (c) 2014, 2016 by Gideon Knocke, Timothe Litt.
-
-This is free software; you can redistribute it and/or modify it under
-the same terms as the Perl 5 programming language system itself.
-
-=head1 BUG REPORTS
-
-Please report any bugs or feature requests on the bugtracker website
-https://rt.cpan.org/Public/Dist/Display.html?Name=Crypt-PKCS10 or by email
-to bug-crypt-pkcs10@rt.cpan.org.
-
-When submitting a bug or request, please include a test-file or a
-patch to an existing test-file that illustrates the bug or desired
-feature.
+GPL v1 -- See LICENSE file for details
 
 =cut
